@@ -2,24 +2,15 @@
 Version numbers exposed by PyPy through the 'sys' module.
 """
 import os
-from rpython.translator.platform import platform
+from rpython.rlib import compilerinfo
 from pypy.interpreter import gateway
 
 #XXX # the release serial 42 is not in range(16)
-CPYTHON_VERSION            = (2, 7, 9, "final", 42)
+CPYTHON_VERSION            = (2, 7, 10, "final", 42)
 #XXX # sync CPYTHON_VERSION with patchlevel.h, package.py
 CPYTHON_API_VERSION        = 1013   #XXX # sync with include/modsupport.h
 
-PYPY_VERSION               = (2, 6, 0, "final", 0)    #XXX # sync patchlevel.h
-
-if platform.name == 'msvc':
-    COMPILER_INFO = 'MSC v.%d 32 bit' % (platform.version * 10 + 600)
-elif platform.cc is not None and \
-        os.path.basename(platform.cc).startswith(('gcc', 'clang')):
-    from rpython.rtyper.tool import rffi_platform
-    COMPILER_INFO = 'GCC ' + rffi_platform.getdefinedstring('__VERSION__', '')
-else:
-    COMPILER_INFO = ""
+PYPY_VERSION               = (2, 6, 1, "final", 0)    #XXX # sync patchlevel.h
 
 
 import pypy
@@ -57,19 +48,24 @@ def get_version_info(space):
     w_version_info = app.wget(space, "version_info")
     return space.call_function(w_version_info, space.wrap(CPYTHON_VERSION))
 
-def get_version(space):
+def _make_version_template(PYPY_VERSION=PYPY_VERSION):
     ver = "%d.%d.%d" % (PYPY_VERSION[0], PYPY_VERSION[1], PYPY_VERSION[2])
     if PYPY_VERSION[3] != "final":
         ver = ver + "-%s%d" %(PYPY_VERSION[3], PYPY_VERSION[4])
-    return space.wrap("%d.%d.%d (%s, %s, %s)\n[PyPy %s%s]" % (
+    template = "%d.%d.%d (%s, %s, %s)\n[PyPy %s with %%s]" % (
         CPYTHON_VERSION[0],
         CPYTHON_VERSION[1],
         CPYTHON_VERSION[2],
         get_repo_version_info(root=pypyroot)[1],
         date,
         time,
-        ver,
-        compiler_version()))
+        ver)
+    assert template.count('%') == 1     # only for the "%s" near the end
+    return template
+_VERSION_TEMPLATE = _make_version_template()
+
+def get_version(space):
+    return space.wrap(_VERSION_TEMPLATE % compilerinfo.get_compiler_info())
 
 def get_winver(space):
     return space.wrap("%d.%d" % (
@@ -111,8 +107,3 @@ def tuple2hex(ver):
             ver[2] << 8    |
             d[ver[3]] << 4 |
             subver)
-
-def compiler_version():
-    if not COMPILER_INFO:
-        return ""
-    return " with %s" % (COMPILER_INFO,)
