@@ -32,10 +32,5 @@ class TestBasic(Jit386Mixin, test_ajit.BaseLLtypeTests):
         res = self.meta_interp(f, [31], enable_opts='')
         assert res == -4
 
-    def test_r_dict(self):
-        # a Struct that belongs to the hash table is not seen as being
-        # included in the larger Array
-        py.test.skip("issue with ll2ctypes")
-
     def test_free_object(self):
         py.test.skip("issue of freeing, probably with ll2ctypes")
