@@ -806,10 +806,14 @@ def import_from_mixin(M, special_methods=['__init__', '__del__']):
     flatten = {}
     caller = sys._getframe(1)
     caller_name = caller.f_globals.get('__name__')
+    immutable_fields = []
     for base in inspect.getmro(M):
         if base is object:
             continue
         for key, value in base.__dict__.items():
+            if key == '_immutable_fields_':
+                immutable_fields.extend(value)
+                continue
             if key.startswith('__') and key.endswith('__'):
                 if key not in special_methods:
                     continue
@@ -835,4 +839,9 @@ def import_from_mixin(M, special_methods=['__init__', '__del__']):
         if key in target:
             raise Exception("import_from_mixin: would overwrite the value "
                             "already defined locally for %r" % (key,))
+        if key == '_mixin_':
+            raise Exception("import_from_mixin(M): class M should not "
+                            "have '_mixin_ = True'")
         target[key] = value
+    if immutable_fields:
+        target['_immutable_fields_'] = target.get('_immutable_fields_', []) + immutable_fields
